from welleng.survey import Survey, make_survey_header
from welleng.clearance import IscwsaClearance
import numpy as np
import json

"""
Test that the ISCWSA clearance model is working within a defined tolerance,
testing against the ISCWSA standard set of wellpaths for evaluating clearance
scenarios using the MWD Rev4 error model.
"""

# Read well and validation data
filename = (
    "tests/test_data/clearance_iscwsa_well_data.json"
)
data = json.load(open(filename))


def generate_surveys(data):
    # Generate surveys for imported wells
    surveys = {}

    for well in data['wells']:
        sh = make_survey_header(data["wells"][well]["header"])

        if well == "Reference well":
            radius = 0.4572
        else:
            radius = 0.3048

        s = Survey(
            md=data["wells"][well]["MD"],
            inc=data["wells"][well]["IncDeg"],
            azi=data["wells"][well]["AziDeg"],
            n=data["wells"][well]["N"],
            e=data["wells"][well]["E"],
            tvd=data["wells"][well]["TVD"],
            radius=radius,
            header=sh,
            error_model="ISCWSA MWD Rev4",
            start_xyz=[
                data["wells"][well]["E"][0],
                data["wells"][well]["N"][0],
                data["wells"][well]["TVD"][0]
                ],
            start_nev=[
                data["wells"][well]["N"][0],
                data["wells"][well]["E"][0],
                data["wells"][well]["TVD"][0]
                ],
            deg=True,
            unit="meters"
        )
        surveys[well] = s

    return surveys


def test_minimize_sf(data=data):
    surveys = generate_surveys(data)
    reference = surveys["Reference well"]
    offset = surveys["09 - well"]

    result = IscwsaClearance(reference, offset, minimize_sf=False)
    result_min = IscwsaClearance(reference, offset, minimize_sf=True)

    idx = np.where(result_min.ref.interpolated == False)

    # Check that interpolated survey is not corrupted
    for attr in [
        'azi_grid_rad', 'azi_mag_rad', 'azi_true_rad', 'cov_hla', 'cov_nev',
        'pos_nev', 'pos_xyz', 'md', 'radius'
    ]:
        assert np.allclose(
            getattr(result.ref, attr), getattr(result_min.ref, attr)[idx]
        )

        pass

    for attr in [
        'Rr', 'calc_hole', 'distance_cc', 'eou_boundary',
        'eou_separation', 'hoz_bearing', 'idx', 'masd', 'off_cov_hla',
        'off_cov_nev', 'off_delta_hlas', 'off_delta_nevs', 'off_pcr',
        'ref_cov_hla', 'ref_cov_nev', 'ref_delta_hlas', 'ref_delta_nevs',
        'ref_nevs', 'ref_pcr', 'sf', 'wellbore_separation'
    ]:
        # `toolface_bearing` and `trav_cyl_azi_deg` are a bit unstable when
        # well paths are parallel.

        assert np.allclose(
            getattr(result, attr), getattr(result_min, attr)[idx],
            rtol=1e-01, atol=1e-02
        )

        pass


def test_clearance_iscwsa(data=data, rtol=1e-02, atol=1e-03):
    surveys = generate_surveys(data)
    reference = surveys["Reference well"]

    # Perform clearance checks for each survey
    for well in surveys:
        if well != "09 - well":
            continue
        if well == "Reference well":
            continue
        else:
            offset = surveys[well]
            # skip well 10
            if well in ["10 - well"]:
                continue
            else:
                for b in [False, True]:
                    result = IscwsaClearance(reference, offset, minimize_sf=b)

                    assert np.allclose(
                        result.sf[np.where(result.ref.interpolated == False)],
                        np.array(data["wells"][well]["SF"]),
                        rtol=rtol, atol=atol
                    )

    pass


# make above test runnanble separately
if __name__ == '__main__':
    test_minimize_sf(data=data)
    test_clearance_iscwsa(data=data)



import welleng as we
import inspect
import sys

survey = we.survey.Survey(
    md=[0, 500, 1000, 2000, 2500, 3500],
    inc=[0, 0, 30, 90, 100, 80],
    azi=[45, 45, 45, 90, 90, 180],
    radius=10,
)


def test_survey_interpolate_survey(step=30):
    global survey
    survey_interp = we.survey.interpolate_survey(survey, step=step)
    assert isinstance(survey_interp, we.survey.Survey)

    survey_interp = survey.interpolate_survey(step=step)
    assert isinstance(survey_interp, we.survey.Survey)


def test_survey_interpolate_survey_tvd(step=10):
    global survey
    survey_interp = survey.interpolate_survey(step=30)
    survey_interp_tvd = we.survey.interpolate_survey_tvd(
        survey_interp, step=step
    )
    assert isinstance(survey_interp_tvd, we.survey.Survey)

    survey_interp_tvd = survey_interp.interpolate_survey_tvd(step=step)
    assert isinstance(survey_interp_tvd, we.survey.Survey)


def test_interpolate_md(md=800):
    global survey
    node = survey.interpolate_md(md=md)
    assert isinstance(node, we.node.Node)


def test_interpolate_tvd(tvd=800):
    global survey
    node = survey.interpolate_tvd(tvd=tvd)
    assert isinstance(node, we.node.Node)


def one_function_to_run_them_all():
    """
    Function to gather the test functions so that they can be tested by
    running this module.

    https://stackoverflow.com/questions/18907712/python-get-list-of-all-
    functions-in-current-module-inspecting-current-module
    """
    test_functions = [
        obj for name, obj in inspect.getmembers(sys.modules[__name__])
        if (inspect.isfunction(obj)
            and name.startswith('test')
            and name != 'all')
    ]

    for f in test_functions:
        f()

        pass


if __name__ == '__main__':
    one_function_to_run_them_all()



import inspect
import sys

from welleng.units import ureg
from welleng.utils import annular_volume


def test_annular_volume():
    av = annular_volume(
        od=ureg('12.25 inch').to('meter'),
        id=ureg(f'{9+5/8} inch').to('meter'),
        length=ureg('1000 meter')
    )

    assert av.m == 3.491531223156194
    assert str(av.u) == 'meter ** 3'

    pass


def one_function_to_run_them_all():
    """
    Function to gather the test functions so that they can be tested by
    running this module.

    https://stackoverflow.com/questions/18907712/python-get-list-of-all-
    functions-in-current-module-inspecting-current-module
    """
    test_functions = [
        obj for name, obj in inspect.getmembers(sys.modules[__name__])
        if (inspect.isfunction(obj)
            and name.startswith('test')
            and name != 'all')
    ]

    for f in test_functions:
        f()

        pass


if __name__ == '__main__':
    one_function_to_run_them_all()



import inspect
import sys

import welleng as we

reference = {
    'x': 588319.02, 'y': 5770571.03, 'northing': 5770571.03,
    'easting': 588319.02, 'latitude': 52.077583926214494,
    'longitude': 4.288694821453205, 'convergence': 1.0166440347220762,
    'scale_factor': 0.9996957469340422, 'magnetic_field_intensity': 49381,
    'declination': 2.213, 'dip': -67.199, 'date': '2023-12-16',
    'srs': 'EPSG:23031'
}


def test_known_location():
    calculator = we.survey.SurveyParameters(reference.get('srs'))
    survey_parameters = calculator.get_factors_from_x_y(
        x=reference.get('x'), y=reference.get('y'), date=reference.get('date')
    )

    for k, v in survey_parameters.items():
        try:
            assert round(v, 3) == round(reference.get(k), 3)
        except TypeError:
            assert v == reference.get(k)

    pass


def one_function_to_run_them_all():
    """
    Function to gather the test functions so that they can be tested by
    running this module.

    https://stackoverflow.com/questions/18907712/python-get-list-of-all-
    functions-in-current-module-inspecting-current-module
    """
    test_functions = [
        obj for name, obj in inspect.getmembers(sys.modules[__name__])
        if (inspect.isfunction(obj)
            and name.startswith('test')
            and name != 'all')
    ]

    for f in test_functions:
        f()

        pass


if __name__ == '__main__':
    one_function_to_run_them_all()
